@@ -17,7 +17,8 @@
 //!
 //! # fn main() -> Result<(), Box<dyn std::error::Error>> {
 //! for card in PcscClient::cards()? {
-//!     let open = Open::open_card(card)?;
+//!     let mut ca = card.into();
+//!     let open = Open::open(&mut ca)?;
 //!     println!("Found OpenPGP card with ident '{}'",
 //!              open.application_identifier()?.ident());
 //! }
@@ -32,8 +33,8 @@
 //! use openpgp_card_sequoia::card::Open;
 //!
 //! # fn main() -> Result<(), Box<dyn std::error::Error>> {
-//! let card = PcscClient::open_by_ident("abcd:12345678")?;
-//! let mut open = Open::open_card(card)?;
+//! let mut ca = PcscClient::open_by_ident("abcd:12345678")?.into();
+//! let mut open = Open::open(&mut ca)?;
 //! # Ok(())
 //! # }
 //! ```
@@ -54,8 +55,8 @@
 //! # fn main() -> Result<(), Box<dyn std::error::Error>> {
 //! // Open card via PCSC
 //! use sequoia_openpgp::policy::StandardPolicy;
-//! let card = PcscClient::open_by_ident("abcd:12345678")?;
-//! let mut open = Open::open_card(card)?;
+//! let mut ca = PcscClient::open_by_ident("abcd:12345678")?.into();
+//! let mut open = Open::open(&mut ca)?;
 //!
 //! // Get authorization for user access to the card with password
 //! open.verify_user("123456")?;
@@ -95,8 +96,8 @@
 //! # fn main() -> Result<(), Box<dyn std::error::Error>> {
 //! // Open card via PCSC
 //! use sequoia_openpgp::policy::StandardPolicy;
-//! let card = PcscClient::open_by_ident("abcd:12345678")?;
-//! let mut open = Open::open_card(card)?;
+//! let mut ca = PcscClient::open_by_ident("abcd:12345678")?.into();
+//! let mut open = Open::open(&mut ca)?;
 //!
 //! // Get authorization for signing access to the card with password
 //! open.verify_user_for_signing("123456")?;
@@ -125,8 +126,8 @@
 //!
 //! # fn main() -> Result<(), Box<dyn std::error::Error>> {
 //! // Open card via PCSC
-//! let card = PcscClient::open_by_ident("abcd:12345678")?;
-//! let mut open = Open::open_card(card)?;
+//! let mut ca = PcscClient::open_by_ident("abcd:12345678")?.into();
+//! let mut open = Open::open(&mut ca)?;
 //!
 //! // Get authorization for admin access to the card with password
 //! open.verify_admin("12345678")?;
@@ -151,4 +152,4 @@ pub mod sq_util;
 pub mod util;
 
 /// Shorthand for Sequoia public key data (a single public (sub)key)
-pub(crate) type PublicKey = Key<key::PublicParts, key::UnspecifiedRole>;
+pub type PublicKey = Key<key::PublicParts, key::UnspecifiedRole>;
