@@ -10,9 +10,8 @@ use openpgp::policy::Policy;
 use openpgp::types::{Curve, PublicKeyAlgorithm};
 use sequoia_openpgp as openpgp;
 
-use openpgp_card::card_app::CardApp;
-use openpgp_card::errors::OpenpgpCardError;
-use openpgp_card::Hash;
+use openpgp_card::crypto_data::Hash;
+use openpgp_card::{CardApp, Error};
 
 use crate::PublicKey;
 
@@ -33,10 +32,10 @@ impl<'a> CardSigner<'a> {
         ca: &'a mut CardApp,
         cert: &openpgp::Cert,
         policy: &dyn Policy,
-    ) -> Result<CardSigner<'a>, OpenpgpCardError> {
+    ) -> Result<CardSigner<'a>, Error> {
         // Get the fingerprint for the signing key from the card.
-        let ard = ca.get_app_data()?;
-        let fps = CardApp::get_fingerprints(&ard)?;
+        let ard = ca.get_application_related_data()?;
+        let fps = ard.get_fingerprints()?;
         let fp = fps.signature();
 
         if let Some(fp) = fp {
@@ -60,19 +59,18 @@ impl<'a> CardSigner<'a> {
 
                 Ok(Self::with_pubkey(ca, public))
             } else {
-                Err(OpenpgpCardError::InternalError(anyhow!(
+                Err(Error::InternalError(anyhow!(
                     "Failed to find a matching (sub)key in cert"
                 )))
             }
         } else {
-            Err(OpenpgpCardError::InternalError(anyhow!(
-                "Failed to get the signing key's Fingerprint \
-                from the card"
+            Err(Error::InternalError(anyhow!(
+                "Failed to get the signing key's Fingerprint from the card"
             )))
         }
     }
 
-    pub fn with_pubkey(
+    pub(crate) fn with_pubkey(
         ca: &'a mut CardApp,
         public: PublicKey,
     ) -> CardSigner<'a> {
@@ -85,17 +83,18 @@ impl<'a> crypto::Signer for CardSigner<'a> {
         &self.public
     }
 
-    /// Delegate a signing operation to the OpenPGP card.
-    ///
-    /// This fn prepares the data structures that openpgp-card needs to
-    /// perform the signing operation.
-    ///
-    /// (7.2.10 PSO: COMPUTE DIGITAL SIGNATURE)
     fn sign(
         &mut self,
         hash_algo: openpgp::types::HashAlgorithm,
         digest: &[u8],
     ) -> openpgp::Result<mpi::Signature> {
+        // Delegate a signing operation to the OpenPGP card.
+        //
+        // This fn prepares the data structures that openpgp-card needs to
+        // perform the signing operation.
+        //
+        // (7.2.10 PSO: COMPUTE DIGITAL SIGNATURE)
+
         match (self.public.pk_algo(), self.public.mpis()) {
             #[allow(deprecated)]
             (PublicKeyAlgorithm::RSASign, mpi::PublicKey::RSA { .. })
